@@ -9,6 +9,7 @@
 """
 from types import FunctionType as Function
 from typing import Any, Callable, Dict, List, Type, TypeVar, Union, cast
+from reprlib import recursive_repr
 
 DataClass = Any  # type hint for variables that should be data class instances
 
@@ -58,8 +59,8 @@ def factory(producer: Callable[[], Factory.Produces]) -> Factory.Produces:
 
 class DataClassMeta(type):
     """The metaclass that implements data class behaviour."""
-    DEFAULT_OPTIONS = dict(init=True, repr=True, eq=True, iter=False, frozen=False, kwargs=False, slots=False,
-                           order=False, unsafe_hash=True, hide_internals=True)
+    DEFAULT_OPTIONS = dict(init=True, repr=True, eq=True, iter=False, frozen=False, order=False, unsafe_hash=False,
+                           kwargs=False, slots=False, hide_internals=True)
 
     def __new__(mcs, name, bases, dict_, **kwargs):
         """Create a new data class."""
@@ -112,11 +113,10 @@ class DataClassMeta(type):
             del dict_['__slots__']
 
         if options['init'] and all_annotations and '__init__' not in all_attrs:
-            dict_.setdefault('__init__', generate_init(all_annotations, all_defaults, post_init,
-                                                       options['kwargs'], options['frozen']))
+            dict_.setdefault('__init__', generate_init(all_annotations, all_defaults, options, post_init))
 
         if options['repr']:
-            '__repr__' in all_attrs or dict_.setdefault('__repr__', __repr__)
+            '__repr__' in all_attrs or dict_.setdefault('__repr__', recursive_repr()(__repr__))
 
         if options['eq']:
             '__eq__' in all_attrs or dict_.setdefault('__eq__', __eq__)
@@ -148,45 +148,6 @@ class DataClassMeta(type):
         cls.__tuple__ = property(eval(f'lambda self: ({tuple_expr})'))
 
 
-def generate_init(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs: bool, frozen: bool) -> Function:
-    """Generate and return an __init__ method for a data class. This method has as parameters all fields of the data
-    class. When the data class is initialised, arguments to this function are applied to the fields of the new instance.
-    A user-defined __init__, if present, must be aliased to avoid conflicting."""
-    arguments = [a for a in annotations if a not in defaults]
-    default_arguments = [f'{a}={a}' for a in defaults]
-    args = ['*args'] if user_init else []
-    kwargs = ['**kwargs'] if user_init or gen_kwargs else []
-
-    parameters = ', '.join(arguments + default_arguments + args + kwargs)
-
-    # surprisingly, given global lookups are slow, using them is the fastest way to compare a field to its default
-    # the alternatives are to look up on self (which wouldn't work when slots=True) or look up self.__defaults__
-    default_names = {f'default_{n}': v for n, v in defaults.items()}
-
-    # determine what to do with arguments before assignment. If the argument matches a mutable default, make a copy
-    references = {n: f'{n}.copy() if {n} is default_{n} else {n}' if n in defaults and hasattr(defaults[n], 'copy')
-                  else n for n in annotations}
-
-    # if the class is frozen, use the necessary but far slower object.__setattr__
-    assignments = [f'object.__setattr__(self, {n!r}, {r})' if frozen
-                   else f'self.{n} = {r}' for n, r in references.items()]
-
-    # generate the function
-    lines = [f'def __init__(self, {parameters}):',
-             *assignments,
-             'self.__post_init__(*args, **kwargs)' if user_init else '']
-
-    return eval_function('__init__', lines, annotations, defaults, default_names)
-
-
-def generate_hash(annotations: Dict) -> Function:
-    """Generate a __hash__ method for a data class. The hashed value consists of a tuple of the instance's type
-    followed by any fields marked as "Hashed"."""
-    hash_of = ', '.join(['self.__class__', *(f'self.{f}' for f, h in annotations.items() if Hashed.is_hinted(h))])
-    return eval_function('__hash__', ['def __hash__(self):',
-                                      f'return hash(({hash_of}))'], dict(self=DataClass), {}, {})
-
-
 def eval_function(name: str, lines: List[str], annotations: Dict, locals_: Dict, globals_: Dict) -> Function:
     """Evaluate a function definition, returning the resulting object."""
     exec('\n\t'.join(lines), globals_, locals_)
@@ -203,6 +164,45 @@ def is_user_func(obj: Any, object_methods=frozenset(vars(object).values())) -> b
     return type(obj) is Function and obj not in object_methods and not hasattr(obj, '__dataclass__')
 
 
+def generate_init(annotations: Dict, defaults: Dict, options: Dict, user_init: bool) -> Function:
+    """Generate and return an __init__ method for a data class. This method has as parameters all fields of the data
+    class. When the data class is initialised, arguments to this function are applied to the fields of the new instance.
+    A user-defined __init__, if present, must be aliased to avoid conflicting."""
+    arguments = [a for a in annotations if a not in defaults]
+    default_arguments = [f'{a}={a}' for a in defaults]
+    args = ['*args'] if user_init else []
+    kwargs = ['**kwargs'] if user_init or options['kwargs'] else []
+
+    parameters = ', '.join(arguments + default_arguments + args + kwargs)
+
+    # surprisingly, given global lookups are slow, using them is the fastest way to compare a field to its default
+    # the alternatives are to look up on self (which wouldn't work when slots=True) or look up self.__defaults__
+    default_names = {f'default_{n}': v for n, v in defaults.items()}
+
+    # determine what to do with arguments before assignment. If the argument matches a mutable default, make a copy
+    references = {n: f'{n}.copy() if {n} is default_{n} else {n}' if n in defaults and hasattr(defaults[n], 'copy')
+                  else n for n in annotations}
+
+    # if the class is frozen, use the necessary but far slower object.__setattr__
+    assignments = [f'object.__setattr__(self, {n!r}, {r})' if options['frozen']
+                   else f'self.{n} = {r}' for n, r in references.items()]
+
+    # generate the function
+    lines = [f'def __init__(self, {parameters}):',
+             *assignments,
+             'self.__post_init__(*args, **kwargs)' if user_init else '']
+
+    return eval_function('__init__', lines, annotations, defaults, default_names)
+
+
+def generate_hash(annotations: Dict[str, Type]) -> Function:
+    """Generate a __hash__ method for a data class. The hashed value consists of a tuple of the instance's type
+    followed by any fields marked as "Hashed"."""
+    hash_of = ', '.join(['self.__class__', *(f'self.{f}' for f, h in annotations.items() if Hashed.is_hinted(h))])
+    return eval_function('__hash__', ['def __hash__(self):',
+                                      f'return hash(({hash_of}))'], dict(self=DataClass), {}, {})
+
+
 # generic method implementations common to all data classes
 from .functions import values, fields
 
@@ -223,8 +223,7 @@ def __iter__(self):
 
 def __repr__(self):
     show_internals = not self.__dataclass__['hide_internals']
-    field_values = ', '.join(f'{f}=...' if v is self
-                             else f'{f}={v!r}' for f, v in values(self, show_internals).items())
+    field_values = ', '.join(f'{f}={v!r}' for f, v in values(self, show_internals).items())
     return f'{type(self).__name__}({field_values})'
 
 
